@@ -39,7 +39,7 @@ from pysollib.stack import \
         StackWrapper, \
         WasteStack, \
         WasteTalonStack
-from pysollib.util import ACE, ANY_RANK, ANY_SUIT, JACK, KING, NO_RANK, \
+from pysollib.util import ACE, ANY_RANK, ANY_SUIT, JACK, KING, \
         QUEEN, RANKS, \
         UNLIMITED_CARDS
 # ************************************************************************
@@ -83,15 +83,14 @@ class Camelot_RowStack(ReserveStack):
             cr = cards[0].rank
             if cr == KING:
                 return self.id in (0, 3, 12, 15)
-            elif cr == QUEEN:
+            if cr == QUEEN:
                 return self.id in (1, 2, 13, 14)
-            elif cr == JACK:
+            if cr == JACK:
                 return self.id in (4, 7, 8, 11)
             return True
-        else:
-            if len(self.cards) == 0:
-                return False
-            return self.cards[-1].rank + cards[0].rank == 8
+        if len(self.cards) == 0:
+            return False
+        return self.cards[-1].rank + cards[0].rank == 8
 
     def canMoveCards(self, cards):
         if not self.game.is_fill:
@@ -105,6 +104,11 @@ class Camelot_RowStack(ReserveStack):
             self.playMoveMove(1, game.s.foundations[0], sound=False)
             self.fillStack()
             return True
+        if not self.cards and game.s.talon.cards and \
+                self.acceptsCards(game.s.talon, [game.s.talon.cards[-1]]):
+            game.s.talon.playMoveMove(1, self)
+            return True
+
         return False
 
     def moveMove(self, ncards, to_stack, frames=-1, shadow=-1):
@@ -262,9 +266,7 @@ class SlyFox_Talon(OpenTalonStack):
         old_state = self.game.enterState(self.game.S_FILL)
         self.game.saveStateMove(2 | 16)            # for undo
         if old_state == self.game.S_PLAY and to_stack in self.game.s.rows:
-            n = self.game.num_dealled
-            if n < 0:
-                n = 0
+            n = max(self.game.num_dealled, 0)
             self.game.num_dealled = (n+1) % 20
         self.game.saveStateMove(1 | 16)            # for redo
         self.game.leaveState(old_state)
@@ -354,9 +356,7 @@ class SlyFox(Game):
     def updateText(self):
         if self.preview > 1:
             return
-        n = self.num_dealled
-        if n < 0:
-            n = 0
+        n = max(self.num_dealled, 0)
         text = str(n)+'/20'
         self.texts.misc.config(text=text)
 
@@ -600,71 +600,6 @@ class GrandmammasPatience(Game):
         p.dump(self.base_rank)
 
 
-# ************************************************************************
-# * Double Line
-# ************************************************************************
-
-class DoubleLine_RowStack(BasicRowStack):
-    def acceptsCards(self, from_stack, cards):
-        if not BasicRowStack.acceptsCards(self, from_stack, cards):
-            return False
-        # this stack accepts any one card from the Waste pile
-        return from_stack is self.game.s.waste
-
-    def getHelp(self):
-        return _('Tableau. Build regardless of rank and suit.')
-
-
-class DoubleLine(Game):
-
-    def createGame(self):
-
-        l, s = Layout(self), self.s
-        h0 = l.YS+3*l.YOFFSET
-        self.setSize(l.XM+10*l.XS, l.YM+l.YS+l.TEXT_HEIGHT+2*h0)
-
-        x, y = l.XM, l.YM
-        s.talon = WasteTalonStack(x, y, self, max_rounds=1)
-        l.createText(s.talon, 's')
-        x += l.XS
-        s.waste = WasteStack(x, y, self)
-        l.createText(s.waste, 's')
-
-        x += l.XS
-        for i in range(4):
-            s.foundations.append(SS_FoundationStack(x, y, self, suit=i % 4))
-            x += l.XS
-        for i in range(4):
-            s.foundations.append(SS_FoundationStack(x, y, self, suit=i,
-                                 base_rank=KING, dir=-1))
-            x += l.XS
-
-        y = l.YM+l.YS+l.TEXT_HEIGHT
-        for i in range(2):
-            x = l.XM
-            for j in range(10):
-                s.rows.append(DoubleLine_RowStack(x, y, self, max_cards=2,
-                              max_move=1, max_accept=1, base_rank=NO_RANK))
-                x += l.XS
-            y += h0
-
-        l.defaultStackGroups()
-
-    def startGame(self):
-        self.startDealSample()
-        self.s.talon.dealRow()
-        self.s.talon.dealCards()
-
-    def fillStack(self, stack):
-        if stack in self.s.rows and not stack.cards:
-            old_state = self.enterState(self.S_FILL)
-            if not self.s.waste.cards:
-                self.s.talon.dealCards()
-            if self.s.waste.cards:
-                self.s.waste.moveMove(1, stack)
-            self.leaveState(old_state)
-
-
 # register the game
 registerGame(GameInfo(280, Camelot, "Camelot",
                       GI.GT_1DECK_TYPE, 1, 0, GI.SL_BALANCED,
@@ -678,5 +613,3 @@ registerGame(GameInfo(623, PrincessPatience, "Princess Patience",
                       GI.GT_2DECK_TYPE, 2, 0, GI.SL_BALANCED))
 registerGame(GameInfo(622, GrandmammasPatience, "Grandmamma's Patience",
                       GI.GT_NUMERICA, 2, 0, GI.SL_MOSTLY_SKILL))
-registerGame(GameInfo(702, DoubleLine, "Double Line",
-                      GI.GT_NUMERICA, 2, 0, GI.SL_BALANCED))
