@@ -24,22 +24,20 @@
 import locale
 import os
 import time
+import tkinter
+import tkinter.font
+import tkinter.ttk as ttk
 import traceback
 
 from pysollib.mfxutil import KwStruct, destruct, kwdefault, openURL
 from pysollib.mygettext import _
 from pysollib.settings import WIN_SYSTEM
+from pysollib.speech import Speech
 from pysollib.ui.tktile.tkcanvas import MfxCanvas
 from pysollib.ui.tktile.tkutil import after, after_cancel
 from pysollib.ui.tktile.tkutil import bind, unbind_destroy
 from pysollib.ui.tktile.tkutil import makeToplevel, setTransient
 
-import six
-from six import PY2
-from six.moves import tkinter
-from six.moves import tkinter_font
-from six.moves import tkinter_ttk as ttk
-
 # ************************************************************************
 # * abstract base class for the dialogs in this module
 # ************************************************************************
@@ -49,6 +47,8 @@ class MfxDialog:  # ex. _ToplevelDialog
     img = {}
     button_img = {}
 
+    num_open = 0
+
     def __init__(self, parent, title="", resizable=False, default=-1):
         self.parent = parent
         self.status = 0
@@ -56,6 +56,9 @@ class MfxDialog:  # ex. _ToplevelDialog
         self.timer = None
         self.buttons = []
         self.accel_keys = {}
+        MfxDialog.num_open += 1
+        if MfxDialog.num_open == 1 and self.parent.app is not None:
+            self.parent.app.unraiseAll()
         self.top = makeToplevel(parent, title=title)
         # self._frame = ttk.Frame(self.top)
         # self._frame.pack(expand=True, fill='both')
@@ -65,7 +68,7 @@ class MfxDialog:  # ex. _ToplevelDialog
         # self.top.wm_maxsize(w-4, h-32)
         bind(self.top, "WM_DELETE_WINDOW", self.wmDeleteWindow)
 
-    def mainloop(self, focus=None, timeout=0, transient=True):
+    def mainloop(self, focus=None, timeout=0, transient=True, geometry=""):
         bind(self.top, "<Escape>", self.mCancel)
         bind(self.top, '<Alt-Key>', self.altKeyEvent)  # for accelerators
         if focus is not None:
@@ -77,7 +80,8 @@ class MfxDialog:  # ex. _ToplevelDialog
             except tkinter.TclError:
                 if traceback:
                     traceback.print_exc()
-                pass
+            if geometry != "":
+                self.top.geometry(geometry)
             if timeout > 0:
                 self.timer = after(self.top, timeout, self.mTimeout)
             try:
@@ -92,6 +96,9 @@ class MfxDialog:  # ex. _ToplevelDialog
         self.top.destroy()
         self.top.update_idletasks()
         self.top = None
+        MfxDialog.num_open -= 1
+        if MfxDialog.num_open == 0 and self.parent.app is not None:
+            self.parent.app.raiseAll()
         self.parent = None
 
     def wmDeleteWindow(self, *event):
@@ -119,9 +126,9 @@ class MfxDialog:  # ex. _ToplevelDialog
             key = event.char
             try:
                 if os.name == 'nt':
-                    key = six.text_type(key, locale.getpreferredencoding())
+                    key = str(key, locale.getpreferredencoding())
                 else:
-                    key = six.text_type(key, 'utf-8')
+                    key = str(key, 'utf-8')
             except Exception:
                 pass
             else:
@@ -155,7 +162,7 @@ class MfxDialog:  # ex. _ToplevelDialog
             separator = ttk.Separator(self._frame)
             separator.pack(side='bottom', fill='x')
         top_frame = ttk.Frame(self._frame)
-        top_frame.pack(side='top', fill='both', expand=1)
+        top_frame.pack(side='top', fill='both', expand=True)
         return top_frame, bottom_frame
 
     def createBitmaps(self, frame, kw):
@@ -215,9 +222,9 @@ class MfxDialog:  # ex. _ToplevelDialog
                 button_img = MfxDialog.button_img.get(s)
             s = s.replace('&', '')
             if button < 0:
-                widget = ttk.Button(frame, text=s, state="disabled")
+                widget = PysolButton(frame, text=s, state="disabled")
             else:
-                widget = ttk.Button(
+                widget = PysolButton(
                     frame, text=s, default="normal",
                     command=lambda self=self, button=button:
                     self.mDone(button))
@@ -268,6 +275,8 @@ class MfxMessageDialog(MfxDialog):
         msg.pack(fill='both', expand=True, padx=kw.padx, pady=kw.pady)
         #
         focus = self.createButtons(bottom_frame, kw)
+        speech = Speech()
+        parent.after(600, lambda: speech.speak(kw.text))
         self.mainloop(focus, kw.timeout)
 
 
@@ -287,8 +296,6 @@ class MfxExceptionDialog(MfxMessageDialog):
                 (ex.errno, ex.strerror, repr(ex.filename))
         else:
             t = str(ex)
-        if PY2:
-            t = six.text_type(t, errors='replace')
         kw.text = text + t
         MfxMessageDialog.__init__(self, parent, title, **kw.getKw())
 
@@ -300,6 +307,7 @@ class MfxExceptionDialog(MfxMessageDialog):
 class PysolAboutDialog(MfxMessageDialog):
     def __init__(self, app, parent, title, **kw):
         self._url = kw['url']
+        self.app = app
         kw = self.initKw(kw)
         MfxDialog.__init__(self, parent, title, kw.resizable, kw.default)
         top_frame, bottom_frame = self.createFrames(kw)
@@ -314,20 +322,39 @@ class PysolAboutDialog(MfxMessageDialog):
 
         # font_name = msg.lookup('TLabel', 'font')
         font_name = 'TkDefaultFont'
-        font = tkinter_font.Font(parent, name=font_name, exists=True)
+        font = tkinter.font.Font(parent, name=font_name, exists=True)
         font = font.copy()
         font.configure(underline=True)
         url_label = ttk.Label(frame, text=kw.url, font=font,
                               foreground='blue', cursor='hand2')
         url_label.pack()
-        url_label.bind('<1>', self._urlClicked)
+        from pysollib.options import calcCustomMouseButtonsBinding
+        url_label.bind(
+            calcCustomMouseButtonsBinding('<{mouse_button1}>'),
+            self._urlClicked
+        )
         #
+
         focus = self.createButtons(bottom_frame, kw)
+
+        self.splashscreen = tkinter.BooleanVar()
+        self.splashscreen.set(app.opt.splashscreen)
+        show_on_start = PysolCheckbutton(bottom_frame,
+                                         variable=self.splashscreen,
+                                         command=self._splashUpdate,
+                                         text=_("Show this on startup"))
+        show_on_start.grid(row=0, column=0, sticky='w',
+                           padx=1, pady=1)
+
+        parent.after(500, lambda: app.speech.speak(kw.text + " " + kw.url))
         self.mainloop(focus, kw.timeout)
 
     def _urlClicked(self, event):
         openURL(self._url)
 
+    def _splashUpdate(self):
+        self.app.opt.splashscreen = self.splashscreen.get()
+
 
 # ************************************************************************
 # *
@@ -341,11 +368,13 @@ class MfxSimpleEntry(MfxDialog):
         self.createBitmaps(top_frame, kw)
         #
         self.value = value
+        labeltext = label
         if label:
             label = ttk.Label(top_frame, text=label, takefocus=0)
             label.pack(pady=5)
         w = kw.get("e_width", 0)    # width in characters
-        self.var = ttk.Entry(top_frame, exportselection=1, width=w)
+        self.var = PysolEntry(top_frame, exportselection=1, width=w,
+                              fieldname=labeltext)
         self.var.insert(0, value)
         self.var.pack(side='top', padx=kw.padx, pady=kw.pady)
         #
@@ -504,7 +533,7 @@ class MfxScrolledCanvas:
     #
     #
 
-    def setTile(self, app, i, force=False):
+    def setTile(self, app, i, scale_method, force=False):
         tile = app.tabletile_manager.get(i)
         if tile is None or tile.error:
             return False
@@ -521,8 +550,19 @@ class MfxScrolledCanvas:
                     tile.color == app.opt.colors['table']):
                 return False
         #
-        if not self.canvas.setTile(tile.filename, tile.stretch,
-                                   tile.save_aspect):
+        stretch = tile.stretch
+        save_aspect = tile.save_aspect
+
+        if scale_method < 0 and stretch:
+            scale_method = app.opt.tabletile_scale_method
+        elif not stretch:
+            scale_method = 0
+
+        if scale_method > 0:
+            stretch = scale_method > 1
+            save_aspect = scale_method > 2
+
+        if not self.canvas.setTile(tile.filename, stretch, save_aspect):
             tile.error = True
             return False
 
@@ -608,7 +648,9 @@ class MfxScrolledCanvas:
         if self.canvas.busy:
             return
         sb = self.hbar
-        if float(first) <= 0 and float(last) >= 1:
+        # TODO - Setting this to .99 takes the scrollbar size into account.
+        # But there is probably a better way to do it.
+        if float(first) <= 0 and float(last) >= .99:
             sb.grid_remove()
             self.hbar_show = False
         else:
@@ -621,7 +663,8 @@ class MfxScrolledCanvas:
         if self.canvas.busy:
             return
         sb = self.vbar
-        if float(first) <= 0 and float(last) >= 1:
+        # TODO - See _setHbar above.
+        if float(first) <= 0 and float(last) >= .99:
             sb.grid_remove()
             self.vbar_show = False
         else:
@@ -751,9 +794,8 @@ class MyPysolScale:
         elif self.variable:
             value = self.variable.get()
         self.value = value
-        self.command = command = None
-        if 'command' in kw:
-            command = kw['command']
+        command = kw.get('command', None)
+        self.command = command
         kw['command'] = self._scale_command
         if 'label' in kw:
             self.label_text = kw['label']
@@ -763,6 +805,13 @@ class MyPysolScale:
         else:
             self.label_text = None
             width = 3
+        if 'fieldname' in kw:
+            self.field_name = kw['fieldname']
+            del kw['fieldname']
+        else:
+            self.field_name = ''
+
+        self.speech = Speech()
 
         # create widgets
         side = 'left'  # 'top'
@@ -772,6 +821,7 @@ class MyPysolScale:
         self.label.pack(side=side, expand=False, fill='x')
         self.scale = ttk.Scale(self.frame, **kw)
         self.scale.pack(side=side, expand=True, fill='both', pady=4)
+        self.scale.bind("<FocusIn>", self._scale_focus)
 
         if self.variable:
             self.variable.trace('w', self._trace_var)
@@ -798,10 +848,15 @@ class MyPysolScale:
         v = self._round(float(value)*self.resolution)
         self._set_text(v)
         self.variable.set(v)
-        if value != self.value and self.command:
-            self.command(value)
+        if value != self.value:
+            self.speech.speak(value)
+            if self.command:
+                self.command(value)
         self.value = value
 
+    def _scale_focus(self, event):
+        self.speech.speak(self.field_name + " " + self.value)
+
     def pack(self, **kw):
         self.frame.pack(**kw)
 
@@ -840,11 +895,31 @@ PysolScale = MyPysolScale
 
 class PysolCombo(ttk.Combobox):
     def __init__(self, master=None, **kw):
+        self.speech = Speech()
+        if 'fieldname' in kw:
+            label = kw['fieldname']
+            del kw['fieldname']
+        else:
+            label = None
+
+        self.value = kw.get('textvariable', tkinter.StringVar())
+        kw['textvariable'] = self.value
+
         self._command = None
         if 'selectcommand' in kw:
             self._command = kw['selectcommand']
             del kw['selectcommand']
         ttk.Combobox.__init__(self, master, **kw)
+
+        if label is not None:
+            self.field_name = label
+        else:
+            self.field_name = ''
+        self.bind('<FocusIn>', self._focus)
+
+        self.bind("<Up>", lambda c: self._cycle_items(c, dir=-1))
+        self.bind("<Down>", lambda c: self._cycle_items(c, dir=1))
+
         self.bind('<<ComboboxSelected>>', self._callback)
 
     def _callback(self, *args):
@@ -853,3 +928,211 @@ class PysolCombo(ttk.Combobox):
         if self._command is not None:
             return self._command(*args)
         return None
+
+    def _cycle_items(self, *args, **kw):
+        direction = kw.get("dir", 0)
+        values = self['values']
+        current = self.current()
+
+        new_index = (current + direction) % len(values)
+        self.current(new_index)
+
+        selected_label = values[new_index]
+        self.speech.speak(selected_label)
+
+        return "break"
+
+    def _focus(self, event):
+        self.speech.speak(self.field_name + " " + _("Select box") + " " +
+                          str(self.value.get()))
+
+
+# ************************************************************************
+# * Other components (speech support)
+# ************************************************************************
+
+class PysolCheckbutton(ttk.Checkbutton):
+    def __init__(self, master=None, **kw):
+        self.speech = Speech()
+        if 'prefixtext' in kw:
+            prefixtext = kw['prefixtext']
+            del kw['prefixtext']
+        else:
+            prefixtext = None
+
+        self.value = kw.get('variable', tkinter.BooleanVar())
+        kw['variable'] = self.value
+        ttk.Checkbutton.__init__(self, master, **kw)
+
+        if 'text' in kw:
+            if prefixtext is not None:
+                self.field_name = prefixtext + " - " + kw['text']
+            else:
+                self.field_name = kw['text']
+        elif prefixtext is not None:
+            self.field_name = prefixtext
+        else:
+            self.field_name = ''
+        self.bind('<FocusIn>', self._focus)
+
+    def _focus(self, event):
+        self.speech.speak(self.field_name + " " + _("Check box") + " " +
+                          str(self.value.get()))
+
+
+class PysolRadiobutton(ttk.Radiobutton):
+    def __init__(self, master=None, **kw):
+        self.speech = Speech()
+        if 'prefixtext' in kw:
+            prefixtext = kw['prefixtext']
+            del kw['prefixtext']
+        else:
+            prefixtext = None
+
+        self.myvalue = kw.get('value', tkinter.StringVar())
+        self.value = kw.get('variable', tkinter.StringVar())
+        kw['variable'] = self.value
+        ttk.Radiobutton.__init__(self, master, **kw)
+
+        if 'text' in kw:
+            if prefixtext is not None:
+                self.field_name = prefixtext + " - " + kw['text']
+            else:
+                self.field_name = kw['text']
+        elif prefixtext is not None:
+            self.field_name = prefixtext
+        else:
+            self.field_name = ''
+        self.bind('<FocusIn>', self._focus)
+
+    def _focus(self, event):
+        self.speech.speak(self.field_name + " " + _("Radio button") + " " +
+                          str(self.value.get() == self.myvalue))
+
+
+class PysolEntry(ttk.Entry):
+    def __init__(self, master=None, **kw):
+        self.speech = Speech()
+        if 'fieldname' in kw:
+            label = kw['fieldname']
+            del kw['fieldname']
+        else:
+            label = None
+
+        self.value = kw.get('textvariable', tkinter.StringVar())
+        kw['textvariable'] = self.value
+        ttk.Entry.__init__(self, master, **kw)
+
+        if label is not None:
+            self.field_name = label
+        else:
+            self.field_name = ''
+        self.bind('<FocusIn>', self._focus)
+
+    def _focus(self, event):
+        self.speech.speak(self.field_name + " " + _("Text box") + " " +
+                          self.value.get())
+
+
+class PysolText(tkinter.Text):
+    def __init__(self, master=None, **kw):
+        self.speech = Speech()
+        if 'fieldname' in kw:
+            label = kw['fieldname']
+            del kw['fieldname']
+        else:
+            label = None
+
+        tkinter.Text.__init__(self, master, **kw)
+
+        if label is not None:
+            self.field_name = label
+        else:
+            self.field_name = ''
+        self.bind('<FocusIn>', self._focus)
+
+    def _focus(self, event):
+        # Get all text content
+        content = self.get("1.0", "end-1c")
+        self.speech.speak(self.field_name + " " + _("Text area") + " " +
+                          content)
+
+
+class PysolButton(ttk.Button):
+    def __init__(self, master=None, **kw):
+        self.speech = Speech()
+        if 'prefixtext' in kw:
+            prefixtext = kw['prefixtext']
+            del kw['prefixtext']
+        else:
+            prefixtext = None
+
+        if 'text' in kw:
+            if prefixtext is not None:
+                self.field_name = prefixtext + " - " + kw['text']
+            else:
+                self.field_name = kw['text']
+        else:
+            self.field_name = ''
+
+        ttk.Button.__init__(self, master, **kw)
+
+        self.bind('<FocusIn>', self._focus)
+
+    def _focus(self, event):
+        self.speech.speak(self.field_name)
+
+
+class PysolSpinbox(ttk.Spinbox):
+    def __init__(self, master=None, **kw):
+        self.speech = Speech()
+        if 'fieldname' in kw:
+            label = kw['fieldname']
+            del kw['fieldname']
+        else:
+            label = None
+
+        self.value = kw.get('textvariable', tkinter.StringVar())
+        kw['textvariable'] = self.value
+        ttk.Spinbox.__init__(self, master, **kw)
+
+        if label is not None:
+            self.field_name = label
+        else:
+            self.field_name = ''
+        self.bind('<FocusIn>', self._focus)
+
+    def _focus(self, event):
+        self.speech.speak(self.field_name + " " + _("Spin box") + " " +
+                          str(self.value.get()))
+
+
+# Hidden screen reader text - use to store screen
+# reader text for other widgets.
+class PysolScreenReaderText(ttk.Label):
+    def __init__(self, parent, text="", **kw):
+        ttk.Label.__init__(self, parent, text=text, **kw)
+
+    def screenreader_text(self):
+        return self.cget("text")
+
+
+class PysolNotebook(ttk.Notebook):
+    def __init__(self, master=None, **kw):
+        self.speech = Speech()
+        ttk.Notebook.__init__(self, master, **kw)
+        self.bind('<FocusIn>', self._focus)
+        self.bind('<<NotebookTabChanged>>', self._focus)
+
+    def _focus(self, event):
+        current_tab_id = self.select()
+        tab_text = self.tab(current_tab_id, "text")
+        content_text = self._get_screenreader_text(
+            self.nametowidget(current_tab_id))
+        self.speech.speak(tab_text + " " + _("Tab") + " " + content_text)
+
+    def _get_screenreader_text(self, frame):
+        for child in frame.winfo_children():
+            if isinstance(child, PysolScreenReaderText):
+                return child.screenreader_text()
+        return ""
